use serde::Serialize;

use super::TransferRecord;

/// Format preferred by the Echo Client when using the Pick-List function.
///
/// Note that this does not export plate info!
/// Exports of this type should combine all transfers between the two actively selected plates.
#[derive(Serialize, Debug)]
pub struct EchoClientTransferRecord {
    #[serde(rename = "SrcWell")]
    pub source_well: String,
    #[serde(rename = "DestWell")]
    pub destination_well: String,
    #[serde(rename = "XferVol")]
    pub volume: f32,
}

impl From<TransferRecord> for EchoClientTransferRecord {
    fn from(value: TransferRecord) -> Self {
        EchoClientTransferRecord {
            source_well: value.source_well,
            destination_well: value.destination_well,
            volume: value.volume,
        }
    }
}
